package cn.com.qmth.examcloud.web.reports;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.slf4j.LoggerFactory;

import ch.qos.logback.classic.Logger;
import ch.qos.logback.classic.LoggerContext;
import ch.qos.logback.classic.encoder.PatternLayoutEncoder;
import ch.qos.logback.classic.spi.ILoggingEvent;
import ch.qos.logback.core.rolling.RollingFileAppender;
import ch.qos.logback.core.rolling.SizeAndTimeBasedRollingPolicy;
import ch.qos.logback.core.util.FileSize;
import ch.qos.logback.core.util.OptionHelper;
import cn.com.qmth.examcloud.web.config.SystemConfig;

/**
 * 报表日志工厂
 *
 * @author WANGWEI
 * @date 2018年11月13日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public class ReportLoggerFactory {

	private static final Map<String, Logger> LOGGERS_HOLDER = new HashMap<>();

	private static final Object LOCK = new Object();

	/**
	 * 刷新
	 *
	 * @author WANGWEI
	 */
	public static void refresh() {

		for (Entry<String, Logger> entry : LOGGERS_HOLDER.entrySet()) {
			Logger value = entry.getValue();
			value.info(" ");
		}

	}

	/**
	 * 获取Logger
	 *
	 * @author WANGWEI
	 * @param reportClass
	 * @return
	 */
	public static Logger getLogger(Class<? extends BaseReport> reportClass) {
		String name = reportClass.getName();
		Logger logger = LOGGERS_HOLDER.get(name);
		if (null != logger) {
			return logger;
		}
		synchronized (LOCK) {
			logger = LOGGERS_HOLDER.get(name);
			if (null != logger) {
				return logger;
			}
			logger = build(reportClass);
			LOGGERS_HOLDER.put(name, logger);
		}
		return logger;
	}

	/**
	 * 构建
	 *
	 * @author WANGWEI
	 * @param report
	 * @return
	 */
	private static Logger build(Class<? extends BaseReport> reportClass) {
		String name = reportClass.getName();
		String simpleName = reportClass.getSimpleName();
		LoggerContext context = (LoggerContext) LoggerFactory.getILoggerFactory();

		Logger logger = context.getLogger("REP-" + name);
		logger.setAdditive(false);
		RollingFileAppender<ILoggingEvent> appender = new RollingFileAppender<>();

		appender.setContext(context);
		appender.setName("REP-" + name);
		appender.setFile(OptionHelper.substVars(
				getReportsLogRootDirectory() + "/" + name + "/" + simpleName + ".log", context));
		appender.setAppend(true);
		appender.setPrudent(false);
		SizeAndTimeBasedRollingPolicy<ILoggingEvent> policy = new SizeAndTimeBasedRollingPolicy<>();
		String fp = OptionHelper.substVars(getReportsLogRootDirectory() + "/" + name + "/"
				+ simpleName + ".%d{yyyyMMddHHmm}.%i.txt", context);

		policy.setMaxFileSize(FileSize.valueOf("32MB"));
		policy.setFileNamePattern(fp);
		policy.setMaxHistory(1024);
		policy.setTotalSizeCap(FileSize.valueOf("32GB"));
		policy.setParent(appender);
		policy.setContext(context);
		policy.start();

		PatternLayoutEncoder encoder = new PatternLayoutEncoder();
		encoder.setContext(context);
		encoder.setPattern("%m");
		encoder.start();

		appender.setRollingPolicy(policy);
		appender.setEncoder(encoder);
		appender.start();

		logger.addAppender(appender);

		return logger;
	}

	/**
	 * 获取报告根目录
	 *
	 * @author WANGWEI
	 * @return
	 */
	private static String getReportsLogRootDirectory() {
		String dataDir = SystemConfig.getDataDir();
		if (!dataDir.endsWith("/")) {
			dataDir += "/";
		}
		return dataDir + "reports";
	}
}